//no more global variables here
//tracks and mediaInfo array are given from parent.tracksSingletonInstance

function getTracksSingleton()
{
	if(parent.tracksSingletonInstance != null)
		return parent.tracksSingletonInstance;
	if(tracksSingletonInstance)
		return tracksSingletonInstance;
	console.log("EXCEPTION : tracksSingletonInstance cannot be located");
	return false;
}

/**
 * Creates inner media for a video and audio sophie frames
 * @param {Object} data - json of the sophie frame
 * @return - video or audio tag for the frame
 */
function buildMediaInnerElement(data){
	var mediaId;
	var element = document.createElement(data.media.type);
	
	if (data.media.id != null)
		mediaId = data.media.id;
	else
		mediaId = data.id + "media";
	element.setAttribute('id', mediaId);
	
	if (data.media.type == "video") {
		element.setAttribute('type', 'video/mp4');
		
	}
	
	element.controls = false;
	element.setAttribute('src', data.media.source);
	
	if (data.media.autoplay == "true") {
		element.setAttribute('autoplay', 'autoplay');
	}

	element.setAttribute('width', '100%');
	element.setAttribute('height', '100%');
	
	var tracksSingleton = getTracksSingleton();
	
	var mediaInfo = tracksSingleton.getMediaInfo(mediaId);
	if(mediaInfo == null)
	{
		mediaInfo = new Array();
		if (data.media.start != null) {
			mediaInfo["start"] = data.media.start;
		}
		else {
			mediaInfo["start"] = 0;
		}
			
		if (data.media.end != null) {
			mediaInfo["end"] = data.media.end;
		}
		else {
			mediaInfo["end"] = data.media.duration;
		} 
	
		mediaInfo["currentTime"] = null;
		
		mediaInfo["looping"] = data.media.looping;
		
		//parent.tracksSingletonInstance
		tracksSingleton.setMediaInfo(mediaId, mediaInfo);
	}
	return element;
}

function buildMediaPoster(data, innerDiv, element)
{
	//for video object only check for other attributes like poster and videoTouch
	if (data.media.type == "video") {
		//check whether video has a defined poster image and create it if necessary
		if (data.media.poster != null) {
			var poster = document.createElement("image");
			poster.setAttribute('id', (data.id + "mediaposter"));
			poster.setAttribute('src', data.media.poster);
			poster.setAttribute('width', data.width);
			poster.setAttribute('height', (data.height));
			
			var posterStyle = " position:absolute;";
			posterStyle += " left: 0px;";
			posterStyle += " top: 0px;";
			poster.setAttribute('style', posterStyle);
			
			//check whether touching on video is also enabled
			//if (data.media.videoTouch == "enabled") {
				//add only play handler for poster
				var doPlayOnPosterClick = function(){
					playMedia(data.id + "media");
				}
				poster.addEventListener('click', doPlayOnPosterClick, false);
			//}
			innerDiv.appendChild(poster);
			//mediaControler.appendChild(poster);
		}
		//if video touch is enabled, create invisible canvas to catch click event over video 
		if (data.media.videoTouch == "enabled") {
			var clickArea = document.createElement("canvas");
			clickArea.width = data.width;
			clickArea.height = data.height;
			
			var clickAreaStyle = " position:absolute;";
			clickAreaStyle += " left: 0px;";
			clickAreaStyle += " top: 0px;";
			clickArea.setAttribute('style', clickAreaStyle);
			
			var clickAreaContext = clickArea.getContext("2d");
			clickAreaContext.fillStyle = "rgba(255,0,0,0)";
			clickAreaContext.fillRect(0, 0, clickArea.width, clickArea.height);
			//add play/pause handlers for video screen
			var doPlayPauseOnVideoAreaClick = function(){
				if (element.paused == true) {
					playMedia(data.id + "media");
				}
				else{ 
					pauseMedia(data.id+ "media");
				}
			}
			
			innerDiv.appendChild(clickArea);
			//mediaControler.appendChild(clickArea);
			//for android and ipad devices add touchstart event listener	
			if(parent.deviceChecker.android || parent.deviceChecker.iphone)
			{
				clickArea.addEventListener('touchstart', doPlayPauseOnVideoAreaClick, false);
			}	
			else//for pc add click event listener
			{
				clickArea.addEventListener('click', doPlayPauseOnVideoAreaClick, false);
			}
			
		}
	}
	
}

/**
 * Build native media controls like Play, Pause, Stop buttons, Slider Bar and Poster Image
 * also adds event listeners for buttons and draw invisible canvas for catching click event over video
 * @param {Object} data - JSON with necessary data
 * @param {Object} element - used to add event listeners of the object 
 */
function buildMediaControls(data, element)
{
	var mediaId = data.id + "media";
	
	var mediaControler = document.createElement("div");
	mediaControler.id = "video_controller";
	var buttonsCount = 0;
	var mediaControlerHeight = 0;
	
	//create Play/Pause buttons
	if (data.media.controls.play) 
	{
		var playButton = document.createElement("img");
		playButton.id = "playButton" + mediaId;
		playButton.src = "../system/play.png";
		playButton.addEventListener('click', doPlay, false);
		playButton.style.cursor = "pointer";
		mediaControler.appendChild(playButton);
		buttonsCount++;
		var pauseButton = document.createElement("img");
		pauseButton.id = "pauseButton" + mediaId;
		pauseButton.src = "../system/pause.png";
		pauseButton.style.display = "none";
		pauseButton.style.cursor = "pointer";
		pauseButton.addEventListener('click', doPause, false);
		mediaControler.appendChild(pauseButton);
		mediaControlerHeight = 32;
	}
	//create Stop button
	if (data.media.controls.stop) 
	{
		var stopButton = document.createElement("img");
		stopButton.id = "stopButton" + mediaId;
		stopButton.src = "../system/stop.png";
		stopButton.addEventListener('click', doStop, false);
		stopButton.style.cursor = "pointer";
		mediaControler.appendChild(stopButton);
		buttonsCount++;
		mediaControlerHeight = 32
	}
	//create slider bar
	if (data.media.controls.slider) 
	{
		var mediaSlider = document.createElement("canvas");
		mediaSlider.width = data.width - buttonsCount * 32;
		mediaSlider.height = 32;
		mediaSlider.id = "mediaSlider" + mediaId;
		mediaSlider.style.cursor = "pointer";
		
		//create and fill media slider context
		var mediaSliderContext = mediaSlider.getContext("2d");
		mediaSliderContext.fillStyle = "rgb(200,0,0)";
		mediaSliderContext.fillRect(0, 8, mediaSlider.width, 16);
		mediaControler.appendChild(mediaSlider);
		
		//add click handler to slider bar to change current time of media
		mediaSlider.addEventListener("click", function(event){
			var buttonString = new String(event.target.id);
			var mediaId = buttonString.substring(11, event.target.id.length);
			
			var tracksSingleton = getTracksSingleton();
			var track = tracksSingleton.getTrack(mediaId + "track");
			
			//if (tracks[mediaId + "track"] == null) {
			if (track == null) {
				//tracks[mediaId + "track"] = new Track(mediaId);
				var track = new Track(mediaId);
				tracksSingleton.setTrack(mediaId + "track", track);
			}
			
			//do not change current time if media is not previously played because error will occur
			//if (tracks[mediaId + "track"].mediaPlayed == true) {
			if (track.mediaPlayed == true) {
				media = document.getElementById(mediaId);//getMediaElementById(mediaId);//
				if (media != null) {
					var mediaInfo = tracksSingleton.getMediaInfo(mediaId);
					//media.currentTime = ((mediaInfo[mediaId]["start"] + ((event.offsetX) / (mediaSlider.width)) * (mediaInfo[mediaId]["end"] - mediaInfo[mediaId]["start"])));
					media.currentTime = ((mediaInfo["start"] + ((event.offsetX) / (mediaSlider.width)) * (mediaInfo["end"] - mediaInfo["start"])));
				}
			}
		});
		mediaControlerHeight = 32;
		
	}
	
	
	//set styles of media controler
	var mediaControlerStyle = "position: absolute;";
	mediaControlerStyle += "bottom: " + data.paddingBottom + "px;";
	mediaControlerStyle += "left: " + data.paddingLeft + "px;";
	mediaControler.setAttribute('style', mediaControlerStyle);
	
	//play button click handler
	function doPlay(event){
		var buttonString = new String(event.target.id);
		var mediaId = buttonString.substring(10, event.target.id.length);
		media = document.getElementById(mediaId);//getMediaElementById(mediaId);//
		playMedia(mediaId);
	};
	
	//pause button click handler
	function doPause(event){
		var buttonString = new String(event.target.id);
		var mediaId = buttonString.substring(11, event.target.id.length);
		media = document.getElementById(mediaId);//getMediaElementById(mediaId);//
		pauseMedia(mediaId);
	};
	
	//stop button click handler
	function doStop(event){
		var buttonString = new String(event.target.id);
		var mediaId = buttonString.substring(10, event.target.id.length);
		media = document.getElementById(mediaId);//getMediaElementById(mediaId);//
		stopMedia(mediaId);
	}
	
	return mediaControler;
}

//used to play media
function playMedia(mediaId, beginOn)
{
	var media = document.getElementById(mediaId);
	if (media == null) {
		//check if media exist in parent because "bookAudio" is child of parent
		media = parent.document.getElementById(mediaId);
		if (media == null) {
			console.log("EXCEPTION : media with define id (" + mediaId + ") doesn't exist");
			return;
		}
	}
	//console.log("playMedia( " + mediaId + " , " + beginOn + ")   Time=" + new Date().getTime());
	//if other media is currently playing then pause it, because of ipad media issues
	var tracksSingleton = getTracksSingleton();
	var currentPlayingMedia = tracksSingleton.getCurrentPlayingMedia();
		
	if(currentPlayingMedia != null && currentPlayingMedia != mediaId)
	{
		pauseMedia(currentPlayingMedia);
	}
	
		
	tracksSingleton.setCurrentPlayingMedia(mediaId);
	
	if (document.getElementById("playButton"+mediaId) != null)
		document.getElementById("playButton"+mediaId).style.display = "none";
	if (document.getElementById("pauseButton"+mediaId))
		document.getElementById("pauseButton"+mediaId).style.display = "inline";
	
	var mediaInfo = tracksSingleton.getMediaInfo(mediaId);
	
	if (beginOn == null) {
		//if we had initialized media earlier by timeline, we start from defined position
		if (mediaInfo["currentTime"] != null) {
			beginOn = mediaInfo["currentTime"];
			mediaInfo["currentTime"] = null;
		}
		else
		{
			beginOn = media.currentTime;
		}
	}
	
	var track = tracksSingleton.getTrack(mediaId+"track");
	if(track == null)
	{
		track = new Track(mediaId);
		tracksSingleton.setTrack(mediaId+"track", track);
	}
	if (beginOn < mediaInfo["start"] || (mediaInfo["end"] != -1 && mediaInfo["end"] < beginOn))
	{	
		track.play(mediaInfo["start"]);
	}
	else
	{
		media.muted = true;
		track.play(beginOn);		
	}
}

function pauseMedia(mediaId)
{
	//set that no media is playing now
	var tracksSingleton = getTracksSingleton();
	tracksSingleton.setCurrentPlayingMedia(null);
	
	var track = tracksSingleton.getTrack(mediaId + "track");
	if (track != null)
		track.pause();
	
	if (document.getElementById("pauseButton"+mediaId) != null)
		document.getElementById("pauseButton"+mediaId).style.display = "none";
	if (document.getElementById("playButton"+mediaId) != null)
		document.getElementById("playButton"+mediaId).style.display = "inline";
}

function stopMedia(mediaId)
{
	var tracksSingleton = getTracksSingleton();
	
	if (tracksSingleton.getCurrentPlayingMedia() == mediaId) {
		tracksSingleton.setCurrentPlayingMedia(null);
	}
	
	var mediaInfo = tracksSingleton.getMediaInfo(mediaId);
	//mediaInfo[mediaId]["currentTime"] = null;
	mediaInfo["currentTime"] = null;
	
	var track = tracksSingleton.getTrack(mediaId + "track");
	if (track != null)
		track.stop();
	
	if (document.getElementById("playButton"+mediaId) != null)
		document.getElementById("playButton"+mediaId).style.display = "inline";
	if (document.getElementById("pauseButton"+mediaId) != null)
		document.getElementById("pauseButton"+mediaId).style.display = "none";
	
	var media = document.getElementById(mediaId);//getMediaElementById(mediaId);//
	if (media.tagName == "VIDEO") {
		var poster = document.getElementById(mediaId + "poster");
		if (poster != null) 
			poster.style.visibility = "visible";
	}
}

/* 
 * Function for drawing slider of media on media time update
 * event listener is added once on first progress catched after first play of media
 */
function updateMediaSlider(event){
	
	var tracksSingleton = getTracksSingleton();
	
	var element = event.target;
	
	var mediaId = element.id;
	var mediaInfo = tracksSingleton.getMediaInfo(mediaId);
	//if (isNaN(mediaInfo[mediaId]["end"])) {
	if (isNaN(mediaInfo["end"])) {
		//mediaInfo[mediaId]["end"] = element.duration;
		mediaInfo["end"] = element.duration;
	}
	//get slider for selected media element
	var mediaSlider = document.getElementById("mediaSlider" + mediaId);//window.frames[parent.getCurrentPageFrameId()].document.getElementById("mediaSlider" + mediaId);
	
	
	//fill media slider context, if slider exist
	if (mediaSlider != null) {
		var mediaSliderContext = mediaSlider.getContext("2d");
		
		var curTime = event.target.currentTime;
		//if (event.target.currentTime <= mediaInfo[mediaId]["start"]){ 
		if (event.target.currentTime <= mediaInfo["start"]) {
			//curTime = mediaInfo[mediaId]["start"];
			curTime = mediaInfo["start"];
		}
		//var currentMoment = ((curTime - mediaInfo[mediaId]["start"]) / (mediaInfo[mediaId]["end"] - mediaInfo[mediaId]["start"])) * (mediaSlider.width);
		var currentMoment = ((curTime - mediaInfo["start"]) / (mediaInfo["end"] - mediaInfo["start"])) * (mediaSlider.width);
		mediaSliderContext.fillStyle = "rgb(0,200,0)";
		mediaSliderContext.fillRect(0, 8, currentMoment, 16);
		mediaSliderContext.fillStyle = "rgb(200,0,0)";
		mediaSliderContext.fillRect(currentMoment, 8, mediaSlider.width, 16);
		
		//if (event.target.currentTime > mediaInfo[mediaId]["end"]) {
		if (event.target.currentTime >= mediaInfo["end"]) {
			stopMedia(mediaId);
			if (mediaInfo["looping"]){
				element.currentTime = mediaInfo["start"];
				playMedia(mediaId);			
			}
		}
	}
}


//Track class is implemented below


function Track(elementId) {
	var track = this,
		media = document.getElementById(elementId);//getMediaElementById(elementId);
	
	var mediaId = elementId;
	media.autobuffer = true;
	media.preload = "none";
	media.load();
	media.muted = true; // makes no difference on iOS :(
  
	/* This is the magic. Since we can't preload, and loading requires a user's
	input. So we bind a touch event to the body, and fingers crossed, the
	user taps. This means we can call play() and immediate pause - which will
	start the download process - so it's effectively preloaded.
	This logic is pretty insane, but forces iOS devices to successfully
	skip an unload audio to a specific point in time.
	First we play, when the play event fires we pause, allowing the asset
	to be downloaded, once the progress event fires, we should have enough
	to skip the currentTime head to a specific point. */
    
	//used to hide poster image after video is played from desired moment
	var afterPlayed = function (){
		//console.log("afterPlayed handler");
		if (media.tagName == "VIDEO") {
			var poster = document.getElementById(mediaId + "poster");
			if (poster != null) 
				poster.style.visibility = "hidden";
		}
		media.addEventListener("timeupdate", updateMediaSlider);
	}
	
	//used to pause video after initial forced play which we use to start loading of media	 
	var force = function () {
		//console.log("force handler");
		media.play();
		media.removeEventListener('play', force, false);
	 	//media.addEventListener('progress', progress, false);
		
		//first time set little delay before hiding poster image because video is not initialized
		setTimeout( function (){ 
						afterPlayed(); 
						media.addEventListener('play', afterPlayed, false) 
					}, 100, this);
	};
  	
	var progress = function () {
		console.log("progress handler");
		media.removeEventListener('progress', progress, false);
		afterPlayed(); 
		media.addEventListener('play', afterPlayed, false) 
		if (track.updateCallback != null)
			track.updateCallback();
		//add event for slider update here		
		
	};
  
	var canplaythrough = function(){
		media.removeEventListener('canplaythrough', canplaythrough, false);
		if (track.updateCallback != null)
			track.updateCallback();
	};
	 
	media.addEventListener('play', force, false);
	//media.addEventListener('progress', progress, false);
	media.addEventListener('canplaythrough', canplaythrough, false);
   
	this.updateCallback = null;
	this.media = media;
	this.playing = false;
	this.mediaPlayed = false;
}

//just draw media to given position without playing media and setting currentTime of media to this position 
var initQueue = new Array();
var isMediaRunning = false;


Track.prototype.preInit = function(position){
	
	var track = this,
		media = this.media,
     	time = position,
		mediaPlayed = this.mediaPlayed;
	
	var tracksSingleton = getTracksSingleton();
	
	if(mediaPlayed == false)
	{
		//get slider for selected media element
		var mediaSlider = document.getElementById("mediaSlider" + media.id);
		
		//fill media slider context, if slider exist
		if (mediaSlider != null) {
			var mediaSliderContext = mediaSlider.getContext("2d");
			
			var mediaInfo = tracksSingleton.getMediaInfo(media.id);
			//mediaInfo[media.id]
			mediaInfo["currentTime"] = position;
			
			//var currentMoment = ((position - mediaInfo[media.id]["start"]) / (mediaInfo[media.id]["end"] - mediaInfo[media.id]["start"])) * (mediaSlider.width);
			var currentMoment = ((position - mediaInfo["start"]) / (mediaInfo["end"] - mediaInfo["start"])) * (mediaSlider.width);
			mediaSliderContext.fillStyle = "rgb(0,200,0)";
			mediaSliderContext.fillRect(0, 8, currentMoment, 16);
			mediaSliderContext.fillStyle = "rgb(200,0,0)";
			mediaSliderContext.fillRect(currentMoment, 8, mediaSlider.width, 16);
		}
	}
	else
	{	
		this.init(position);		
	}
}

Track.prototype.init = function (position)
{	
	var track = this,
		media = this.media,
     	time = position;
      
	track.playing = true;
	
	media.muted = true;
	media.pause();
	try 
	{
		if (time == 0) 
			time = 0.01; // yay hacks. Sometimes setting time to 0 doesn't play back
		
		media.currentTime = time;
	}
	catch (e) 
	{
		//console.log("exception at Track.prototype.init | e = " + e);
		media.play();		
	
		var progress = function(){
			media.currentTime = time;
			media.pause();	
			media.removeEventListener('progress', progress, false);	
			
		};
		media.addEventListener('progress', progress, false);
	}
};


Track.prototype.play = function (position) {
	var track = this,
		media = this.media,
     	time = position,
		mediaPlayed = this.mediaPlayed;
		
	//track.playing = true;
	

	media.muted = true;
	media.pause();

	try 
	{
		if (time == 0) 
			time = 0.01; // yay hacks. Sometimes setting time to 0 doesn't play back
		media.currentTime = time;
		media.play(); 
		media.muted = false;
		track.mediaPlayed = true;
		track.playing = true;
	}
	catch (e) 
	{
		//if error is handled, try to load media again
		//console.log("exception at Track.prototype.play | e = " + e);
		media.load();
		
		this.updateCallback = function ()
		{
			//console.log("updateCallback at Track.prototype.play | catch e = " + e + " | Time=" + new Date().getTime());
			if (media.currentTime < time) {
				media.currentTime = time;
			}
			media.play();
			this.playing = true;
			this.updateCallback = null;
			
		};
		media.play();
		track.playing = true;
		media.muted = false;
	}
	

};

Track.prototype.pause = function () {
	
	console.log("Track.prototype.pause | Time=" + new Date().getTime());
	
	var track = this,
		media = this.media;
	if (track.playing) {
		track.playing = false;
		media.muted = true;
		media.pause();
	}
}

Track.prototype.stop = function() {
	
	console.log("Track.prototype.stop | Time=" + new Date().getTime());
	
	var track = this;
	if (track.playing)
		track.pause();
	
	try 
	{
		this.media.currentTime = 0.01;
	} 
	catch (e) 
	{
		//console.log("exception at Track.prototype.stop | e = " + e + " | Time=" + new Date().getTime());
		this.updateCallback = function(){
			track.updateCallback = null;
			this.media.currentTime = 0.01;
		};
	}
}


