/**
 * Functions for creation and manipulation of text objects
 */

/**
 * Creates div for text segment of the text frame
 * @param {Object} segmentData - json of the segment
 * @param {Object} frameWidth - width of the text frame
 */
function createTextSegment(segmentData, frameWidth)
{
	var segmentDiv = document.createElement("div");
	var segmentStyle = "vertical-align:baseline; top:";
	segmentStyle += segmentData.location.y + "px;";
	segmentStyle += "left: " + segmentData.location.x + "px;";
	segmentStyle += "position: absolute;";
	segmentStyle += "overflow: hidden;"
	segmentStyle += "width: " + (frameWidth+15) + "px;";
	segmentDiv.setAttribute("style", segmentStyle);
	
	//envelop <font> tags in <pre> tag to enable tabulation rendering
	segmentDiv.appendChild( document.createElement("pre") );
	
	var currentPartJson;
	//concatenate all parts of the segment into common span 
	for (var j = 0; j < segmentData.parts.length; j++)
	{
		currentPartJson = segmentData.parts[j];
		//create separate span tag for each part
		var partFont = document.createElement("font");
		var textStyle = getStyle(currentPartJson.style);
		textStyle += "overflow: hidden;"
		textStyle += "margin-left: " + currentPartJson.xLocation + "px;";
		partFont.setAttribute("style", textStyle);
		if (currentPartJson.id != null)
		{
			partFont.setAttribute('id', currentPartJson.id);
			//partFont.style.cursor = "pointer";
		}
		//add tab symbol &#09 instead of \u0009 code exported from sophie 
		partFont.innerHTML = currentPartJson.text.replace("\u0009", "&#09;");;
						
		//add parts of the segment in <pre> tag to be sure that tablutaion will be rendered
		segmentDiv.children[0].appendChild(partFont);
	}
	return segmentDiv;
}
 
/** 
 * Creating style of a textrun
 * @param {Object} item - part of text from a sophie frame
 */
function getStyle(itemStyle) {
	style = "";
	if (itemStyle.fontfamily != null)
	{
		style = style + "font-family:" + itemStyle.fontfamily + ";";
	}
	else
	{
		style = style + "font-family: Monospace;";
	}
	
	if (itemStyle.fontsize != null)
	{
		style = style + "font-size: " + itemStyle.fontsize + ";";
	}
	else
	{
		style = style + "font-size: 12;";
	}
	
	if (itemStyle.bold == true)
	{
		style = style + "font-weight: bold;";
	}
	
	if (itemStyle.italic == true)
	{
		style = style + "font-style: italic;";
	}
	
	if (itemStyle.foregroundcolor != null)
	{
		style = style + "color: " + itemStyle.foregroundcolor + ";";
	}
	
	if (itemStyle.backgroundcolor != null)
	{
		style = style + "background-color: " + itemStyle.backgroundcolor + ";";
	}
	
	if (itemStyle.underlined == true && itemStyle.strikethrough == true)
	{
		style = style + "text-decoration: underline line-through;";
	}
	else
	{
		if (itemStyle.underlined == true)
		{
			style = style + "text-decoration:underline;";
		}
		
		if (itemStyle.strikethrough == true)
		{
			style = style + "text-decoration:line-through;";
		}
	}
	//alert("style created");
	return style;
}