/**
 * @author tancheto
 */
// Scroller of the thumbnail explorer
var myScroll;

// Array with url-s of the thumbails for each page
var thumbnails = new Array();

// Width of the thumbnail
var thumbWidth = 0;
var thumbnailExplorerWidth = 0;
// Count of the thumbs
var thumbCount = 0;

// Timer for closing of the bottom toolbar
var toolbarTimer;

var thumbnailExisting = true;

var thumbnailPinned = false;

/**
 * Builder for the thumbs of the thumbnail explorer and setting custom style of it
 * @param {Object} pagesCount - count of pages (thumbnails) of the book
 * @param {Object} width - page width
 * @param {Object} height - page height
 */
function buildThumbnailExplorer(pagesCount, width, height)
{
	if (!thumbnailExisting)
		return;
	thumbCount = pagesCount;
	var widthOfThumb = parseInt(width / height * 46);
	thumbWidth = widthOfThumb;
	thumbnailExplorerWidth = widthOfThumb*pagesCount + 10*(pagesCount - 1) + 4;
	document.getElementById("thumbnailExplorer").style.width = thumbnailExplorerWidth;
	document.getElementById("thumbnails").style.width = thumbnailExplorerWidth;
		
	var currentThumb;
	for (var i = 0; i < pagesCount; i++)
	{
		currentThumb = buildThumb(i+1);
		document.getElementById("thumbnails").appendChild(currentThumb);
	}
	
	myScroll = new iScroll("wrapper");
	window.addEventListener("THUMBNAIL_EXPLORER_SCROLLED", refreshThumbnailImages);
	window.addEventListener("THUMBNAIL_EXPLORER_RESIZED", resizeThumbnailExplorer);
	
	updateExplorerWidth();
	
	if (deviceChecker.iphone || deviceChecker.android)
		document.getElementById("pinButton").addEventListener("touchstart", pinExplorer);
	else
		document.getElementById("pinButton").onclick = pinExplorer;
}

/**
 * Build thumb
 * @param {Object} thumbNumber - number of the thumb
 */
function buildThumb(thumbNumber)
{
	var thumbLi = document.createElement("li");
	thumbLi.setAttribute("id", "thumb" + thumbNumber);
	if (deviceChecker.iphone || deviceChecker.android)
		thumbLi.onclick = thumbClicked;
	else
		thumbLi.ondblclick = thumbClicked;
	thumbLi.setAttribute("class", "notCurrentThumb");
	thumbLi.setAttribute("style", "background-size: " + thumbWidth + "px 46px;");
	thumbLi.style.width = thumbWidth;
	if (thumbNumber == 1)
		thumbLi.style.marginLeft = 0;
	if (thumbNumber == thumbCount)
		thumbLi.style.marginRight = 0;
	var thumbSpan = document.createElement("span");
	thumbSpan.setAttribute("id", "span" + thumbNumber);
	thumbSpan.setAttribute("class", "thumbSpan");
	if (deviceChecker.iphone || deviceChecker.android)
		thumbSpan.onclick = spanClicked;
	else
		thumbSpan.ondblclick = spanClicked;
	thumbSpan.style.width = thumbWidth;
	thumbSpan.innerHTML = thumbNumber;
	thumbLi.appendChild(thumbSpan);
	return thumbLi;
}

/**
 * Returns event with which thumbs should be pressed
 */
function getEventForDevice()
{
	if (deviceChecker.android/* || deviceChecker.iphone*/)
		return "touchstart";
	if (deviceChecker.iphone)
		return "click";
	return "dblclick";
}

/**
 * Event handler for clicking on the thumb
 * @param {Object} event - click event
 */
function thumbClicked(event)
{
	event.stopPropagation();
	var target = new String(event.target.id);
	var pageNumber = target.substring(5, target.length);
	goToPage(pageNumber);
	
	resetTimer();
}

/**
 * Event handler for clicking on the label of the thumb
 * @param {Object} event - click event
 */
function spanClicked(event)
{
	event.stopPropagation();
	var target = new String(event.target.id);
	var pageNumber = target.substring(4, target.length);
	goToPage(pageNumber);
	
	resetTimer();
}

/**
 * Scrolling thumbnail explorer to the left
 */
function scrollToLeft() {
	myScroll.scrollTo(-window.innerWidth/2, 0, 100, true);
	
	resetTimer();
}

/**
 * Scrolling thumbnail explorer to the right
 */
function scrollToRight() {
	myScroll.scrollTo(window.innerWidth/2, 0, 100, true);
	
	resetTimer();
}

/**
 * Changing current thumb
 * @param {Object} newThumbNumber - number of the new current thumb
 * @param {Object} oldThumbNumber - number of the old current thumb
 */
function setCurrentThumb(newThumbNumber, oldThumbNumber)
{
	if (!thumbnailExisting)
		return;
	if (oldThumbNumber != -1)
		document.getElementById("thumb" + (oldThumbNumber+1)).setAttribute("class", "notCurrentThumb");
	var newThumbId = "thumb" + (newThumbNumber+1);
	document.getElementById(newThumbId).setAttribute("class", "currentThumb");
	
	myScroll.scrollToElement('li:nth-child('+(newThumbNumber+1)+')', 100);
}

/**
 * Builds images of the visible thumbs on the thumbnail explorer
 */
function buildThumbnailImages()
{
	if (!thumbnailExisting)
		return;
	var firstVisibleThumbNumber = Math.max(-parseInt((myScroll.x + 10) / (thumbWidth + 10)) + 1, 1);
	var countOfVisibleThumbs = Math.min(parseInt((window.innerWidth) / (thumbWidth + 10)) + 1, thumbCount);
	var thumbImage;
	for (var i = firstVisibleThumbNumber; i <= firstVisibleThumbNumber + countOfVisibleThumbs; i++)
	{
		if (document.getElementById("thumb"+i) &&
				(document.getElementById("thumb"+i).style.backgroundImage == "" ||
				document.getElementById("thumb"+i).style.backgroundImage == "none"))
			document.getElementById("thumb"+i).style.backgroundImage = "url("+thumbnails[i-1]+")";
	}
	for (var i = 1; i < firstVisibleThumbNumber; i++)
		document.getElementById("thumb"+i).style.backgroundImage = "none";
	for (var i = firstVisibleThumbNumber + countOfVisibleThumbs + 1; i <= thumbCount; i++)
		document.getElementById("thumb"+i).style.backgroundImage = "none";
}

/**
 * Refreshing images on the thumbnail explorer
 * @param {Object} event - THUMBNAIL_EXPLORER_SCROLLED event
 */
function refreshThumbnailImages(event)
{
	buildThumbnailImages();
	resetTimer();
}

function resizeThumbnailExplorer(event)
{
	updateExplorerWidth();
	buildThumbnailImages();
}

/**
 * Function for visualization of the bottom toolbar
 * @param {Object} event - click event
 */
function openBottomToolbar(event)
{
	document.getElementById("expandToolbarButton").style.display = "none";
	document.getElementById("bottomToolbar").style.display = "block";
	document.getElementById("thumbnailBottomPanel").style.visibility = "visible";
	
	resetTimer();
}


/**
 * Function for displaying about Sophie project dialog
 * @param {Object} event - click event
 */
function openAboutSophieDialog(event)
{
	document.getElementById("aboutSophieDialog").style.display = "block";
	document.getElementById("aboutSophieDialog").style.visibility = "visible";
	parent.centerAboutSophieDialog();
}
/**
 * Function for closing about Sophie project dialog
 * @param {Object} event - click event
 */
function closeAboutSophieDialog(event)
{
	console.log("closeAboutSophieDialog");
	document.getElementById("aboutSophieDialog").style.visibility = "hidden";
}

/**
 * Function for closing of the bottom toolbar
 * @param {Object} event - timer finished event
 */
function closeBottomToolbar(event)
{	
	document.getElementById("expandToolbarButton").style.display = "block";
	document.getElementById("bottomToolbar").style.display = "none";
	document.getElementById("thumbnailBottomPanel").style.visibility = "hidden";
	
	if (toolbarTimer != null)
		clearTimeout(toolbarTimer);
}

/**
 * Reseting timer for closing of the bottom toolbar
 */
function resetTimer()
{
	if (thumbnailPinned)
		return;
	if (toolbarTimer != null)
		clearTimeout(toolbarTimer);
	toolbarTimer = setTimeout("closeBottomToolbar(event)", 5000);
}

function updateExplorerWidth()
{
	if (window.innerWidth >= thumbnailExplorerWidth)
	{
		document.getElementById("leftThumbnailButton").style.width = 0;
		document.getElementById("rightThumbnailButton").style.width = 0;
		document.getElementById("leftThumbnailButtonImage").style.width = 0;
		document.getElementById("rightThumbnailButtonImage").style.width = 0;
		document.getElementById("wrapper").style.left = (window.innerWidth - thumbnailExplorerWidth)/2;
		document.getElementById("wrapper").style.right = 0;
	}
	else
	{
		if (deviceChecker.iphone || deviceChecker.android) {
			document.getElementById("leftThumbnailButton").style.width = 0;
			document.getElementById("rightThumbnailButton").style.width = 0;
			document.getElementById("leftThumbnailButtonImage").style.width = 0;
			document.getElementById("rightThumbnailButtonImage").style.width = 0;
			document.getElementById("wrapper").style.left = 0;
			document.getElementById("wrapper").style.right = 0;
		}
		else {
			document.getElementById("leftThumbnailButton").style.width = 50;
			document.getElementById("rightThumbnailButton").style.width = 50;
			document.getElementById("leftThumbnailButtonImage").style.width = 30;
			document.getElementById("rightThumbnailButtonImage").style.width = 30;
			document.getElementById("wrapper").style.left = 50;
			document.getElementById("wrapper").style.right = 50;
		}
		myScroll.refresh();
	}

}

/**
 * Removes bottom toolbar with thumbnail explorer and buttons for social sharing
 */
function removeBottomToolbar()
{
	document.body.removeChild(document.getElementById("expandToolbarButton"));
	document.body.removeChild(document.getElementById("bottomToolbar"));
	document.body.removeChild(document.getElementById("thumbnailBottomPanel"));
	thumbnailExisting = false;
}

function pinExplorer()
{
	if (!thumbnailPinned)
	{
		thumbnailPinned = true;
		if (toolbarTimer != null) 
			clearTimeout(toolbarTimer);
		var curPinSrc = new String(document.getElementById("pinButton").src);
		document.getElementById("pinButton").src = curPinSrc.replace("icnPin", "icnUnpin");
		document.getElementById("pinButton").title = "Unlock panel";
	}
	else
	{
		
		thumbnailPinned = false;
		resetTimer();
		var curPinSrc = new String(document.getElementById("pinButton").src);
		document.getElementById("pinButton").src = curPinSrc.replace("icnUnpin", "icnPin");
		document.getElementById("pinButton").title = "Lock panel";
	}
}
